///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///
/// =========================================================================
#include "basis_symbolic.h"
#include <sstream>
using namespace rheolef;
using namespace std;
using namespace GiNaC;

static 
void
put_gpl (ostream& out)
{
    out << "///" << endl
	<< "/// This file is part of Rheolef." << endl
	<< "///" << endl
	<< "/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>" << endl
	<< "///" << endl
	<< "/// Rheolef is free software; you can redistribute it and/or modify" << endl
	<< "/// it under the terms of the GNU General Public License as published by" << endl
	<< "/// the Free Software Foundation; either version 2 of the License, or" << endl
	<< "/// (at your option) any later version." << endl
	<< "///" << endl
	<< "/// Rheolef is distributed in the hope that it will be useful," << endl
	<< "/// but WITHOUT ANY WARRANTY; without even the implied warranty of" << endl
	<< "/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the" << endl
	<< "/// GNU General Public License for more details." << endl
	<< "///" << endl
	<< "/// You should have received a copy of the GNU General Public License" << endl
	<< "/// along with Rheolef; if not, write to the Free Software" << endl
	<< "/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA" << endl
	<< "///" << endl
	<< "/// =========================================================================" << endl
	;
}
ex
basis_symbolic_nodal_on_geo::indexed_symbol (const ex& expr0) const
{
    size_type d = dimension();
    ex expr = expr0;
    // first optimize;
    if (d > 0) expr = collect(expr,x);
    if (d > 1) expr = collect(expr,y);
    if (d > 2) expr = collect(expr,z);
    // then subst symbols:
    if (d > 0) expr = expr.subs(x == symbol("hat_x[0]"));
    if (d > 1) expr = expr.subs(y == symbol("hat_x[1]"));
    if (d > 2) expr = expr.subs(z == symbol("hat_x[2]"));
    return expr;
}
void
basis_symbolic_nodal_on_geo::put_cxx_header(ostream& out) const
{
  if (size() == 0) return;
  stringstream class_name;
  class_name << "basis_" << name() << "_" << _hat_K.name();
  out << "template<class T>" << endl
      << "class " << class_name.str() << " {" << endl
      << "public:" << endl
      << "  typedef basis_rep<T>              base;" << endl
      << "  typedef typename base::size_type size_type;" << endl
      << "  static void eval      (const point_basic<T>& hat_x, vector<T>& values);" << endl
      << "  static void eval      (const point_basic<T>& hat_x, typename vector<T>::iterator values);" << endl
      << "  static void grad_eval (const point_basic<T>& hat_x, vector<point_basic<T> >& values);" << endl
      << "  static void grad_eval (const point_basic<T>& hat_x, typename vector<point_basic<T> >::iterator values);" << endl
      << "  static void hat_node  (vector<point_basic<T> >& hat_node);" << endl
      << "  static void hat_node  (typename vector<point_basic<T> >::iterator values);" << endl
      << "};" << endl;
}
void
basis_symbolic_nodal_on_geo::put_cxx_body(ostream& out) const
{
  if (size() == 0) return;
  size_type d = dimension();
  stringstream class_name;
  class_name << "basis_" << name() << "_" << _hat_K.name();
  // --------------------------------------------------
  // eval
  // --------------------------------------------------
  out << "template<class T>" << endl
      << "void" << endl
      << class_name.str() << "<T>::eval(" << endl
      << "  const point_basic<T>&        hat_x," << endl
      << "  typename vector<T>::iterator values)" << endl
      << "{" << endl;
  for (size_type i = 0; i < _basis.size(); i++) {
    ex expr = indexed_symbol (_basis[i]);
    stringstream idx_stream;
    out << "  values[" << i << "] = ";
    expr.print(print_csrc_double(cout));
    out << ";" << endl;
  }
  out << "}" << endl
      << "template<class T>" << endl
      << "void" << endl
      << class_name.str() << "<T>::eval(" << endl
      << "  const point_basic<T>& hat_x," << endl
      << "  vector<T>&            values)" << endl
      << "{" << endl
      << "  values.resize(" << _basis.size() << ");" << endl
      << "  eval (hat_x, values.begin());" << endl
      << "}" << endl;
  // --------------------------------------------------
  // grad_eval 
  // --------------------------------------------------
  out << "template<class T>" << endl
      << "void" << endl
      << class_name.str() << "<T>::grad_eval(" << endl
      << "  const point_basic<T>&                      hat_x," << endl
      << "  typename vector<point_basic<T> >::iterator values)" << endl
      << "{" << endl;
  for (size_type i = 0; i < _basis.size(); i++) {
    for (size_type j = 0; j < d; j++) {
      ex expr = indexed_symbol (_grad_basis[i][j]);
      out << "  values[" << i << "][" << j << "] = ";
      expr.print(print_csrc_double(cout));
      out << ";" << endl;
    }
  }
  out << "}" << endl
      << "template<class T>" << endl
      << "void" << endl
      << class_name.str() << "<T>::grad_eval(" << endl
      << "  const point_basic<T>&    hat_x," << endl
      << "  vector<point_basic<T> >& values)" << endl
      << "{" << endl
      << "  values.resize(" << _basis.size() << ");" << endl
      << "  grad_eval (hat_x, values.begin());" << endl
      << "}" << endl;
  // --------------------------------------------------
  // hat_node
  // --------------------------------------------------
  out << "template<class T>" << endl
      << "void" << endl
      << class_name.str() << "<T>::hat_node (typename vector<point_basic<T> >::iterator x)" << endl
      << "{" << endl;
  for (size_type i = 0; i < _basis.size(); i++) {
    out << "  x[" << i << "] = point_basic<T>(";
    for (size_type j = 0; j < d; j++) {
      ex expr = indexed_symbol (_node[i][j]);
      expr.print(print_csrc_double(cout));
      if (j != d-1) out << ", ";
    }
    out << ");" << endl;
  }
  out << "}" << endl
      << "template<class T>" << endl
      << "void" << endl
      << class_name.str() << "<T>::hat_node (vector<point_basic<T> >& x)" << endl
      << "{" << endl
      << "  x.resize(" << _basis.size() << ");" << endl
      << "  hat_node (x.begin());" << endl
      << "}" << endl;
}
void
basis_symbolic_nodal::put_cxx_header(ostream& out) const
{
    stringstream class_name;
    class_name << "basis_" << name();

    out << "// file automatically generated by \"" << __FILE__ << "\"" << endl;
    put_gpl(out);
    out << "#ifndef _RHEOLEF_" << name() << "_H" << endl
        << "#define _RHEOLEF_" << name() << "_H" << endl
        << "#include \"rheolef/basis.h\"" << endl
        << "#include \"rheolef/tensor.h\"" << endl
        << "namespace rheolef {" << endl
        << endl
        << "template<class T>" << endl
        << "class " << class_name.str() << ": public basis_rep<T> {" << endl
        << "public:" << endl
        << "  typedef typename basis_rep<T>::size_type size_type;" << endl
        << "  " << class_name.str() << "() : basis_rep<T>(\"" << name() << "\") {}" << endl
        << "  ~" << class_name.str() << "();" << endl
        << "  size_type degree () const;" << endl
        << "  size_type size (reference_element hat_K) const;" << endl
        << "  void eval(" << endl
        << "    reference_element                 hat_K," << endl
        << "    const point_basic<T>&             hat_x," << endl
        << "    std::vector<T>&                   values) const;" << endl
        << "  void eval(" << endl
        << "    reference_element                 hat_K," << endl
        << "    const point_basic<T>&             hat_x," << endl
        << "    typename std::vector<T>::iterator values) const;" << endl
        << "  void grad_eval(" << endl
        << "    reference_element                               hat_K," << endl
        << "    const point_basic<T>&                           hat_x," << endl
        << "    std::vector<point_basic<T> >&                   values) const;" << endl
        << "  void grad_eval(" << endl
        << "    reference_element                               hat_K," << endl
        << "    const point_basic<T>&                           hat_x," << endl
        << "    typename std::vector<point_basic<T> >::iterator values) const;" << endl
        << "  void hat_node(" << endl
        << "    reference_element                               hat_K," << endl
        << "    std::vector<point_basic<T> >&                   hat_node) const;" << endl
        << "  void hat_node(" << endl
        << "    reference_element                               hat_K," << endl
        << "    typename std::vector<point_basic<T> >::iterator hat_node) const;" << endl
        << "};" << endl
        << "} // namespace rheoef" << endl
        << "#endif // _RHEOLEF_" << name() << "_H" << endl
        << endl;
}
void
basis_symbolic_nodal::put_cxx_body(ostream& out) const
{
    out << "// file automatically generated by \"" << __FILE__ << "\"" << endl;
    put_gpl(out);
    out << "#include \"" << name() << ".h\"" << endl;
    out << "namespace rheolef {" << endl;
    out << "using namespace std;" << endl;
    for (size_type i = 0; i < reference_element::max_variant; i++) {
      operator[](i).put_cxx_header(out);
    }
    for (size_type i = 0; i < reference_element::max_variant; i++) {
      operator[](i).put_cxx_body(out);
    }
    stringstream class_name;
    class_name << "basis_" << name();

    // --------------------------------------------------
    // destructor
    // --------------------------------------------------
    out << "template<class T>" << endl
        << class_name.str() << "<T>::~" << class_name.str() << "()" << endl
        << "{" << endl
        << "}" << endl;
    // --------------------------------------------------
    // degree
    // --------------------------------------------------
    out << "template<class T>" << endl
        << "typename " << class_name.str() << "<T>::size_type" << endl
	<< class_name.str() << "<T>::degree () const" << endl
        << "{" << endl
        << "    return " << degree() << ";" << endl
        << "}" << endl;
    // --------------------------------------------------
    // size
    // --------------------------------------------------
    out << "template<class T>" << endl
        << "typename " << class_name.str() << "<T>::size_type" << endl
	<< class_name.str() << "<T>::size (" << endl
	<< "    reference_element hat_K) const" << endl
        << "{" << endl
        << "    switch (hat_K.variant()) {" << endl;
    for (size_type i = 0; i < reference_element::max_variant; i++) {
      const basis_symbolic_nodal_on_geo& b = operator[](i);
      if (b.size() == 0) continue;
      out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
          << "        return " << b.size() << ";" << endl
          << "      }" << endl;
    }
    out << "      default : {" << endl
        << "        error_macro (\"size: unsupported `\" << hat_K.name() << \"' element type\");" << endl
        << "        return 0;" << endl
        << "      }" << endl
        << "    }" << endl
        << "}" << endl;
    // --------------------------------------------------
    // eval
    // --------------------------------------------------
    for (size_t k = 0; k <= 1; ++k) {
      string value_t = (k==0) ? "vector<T>&" : "typename vector<T>::iterator";
      out << "template<class T>" << endl
          << "void" << endl
          << class_name.str() << "<T>::eval(" << endl
          << "    reference_element     hat_K," << endl
          << "    const point_basic<T>& hat_x," << endl
          << "    "<< value_t<<"        values) const" << endl
          << "{" << endl
          << "    switch (hat_K.variant()) {" << endl;
      for (size_type i = 0; i < reference_element::max_variant; i++) {
        const basis_symbolic_nodal_on_geo& b = operator[](i);
        if (b.size() == 0) continue;
        out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
            << "        return " << class_name.str() << "_" << b.hat_K().name() << "<T>::eval (hat_x, values);" << endl
            << "      }" << endl;
      }
      out << "      default : {" << endl
          << "        error_macro (\"eval: unsupported `\" << hat_K.name() << \"' element type\");" << endl
          << "      }" << endl
          << "    }" << endl
          << "}" << endl;
    }
    // --------------------------------------------------
    // grad_eval
    // --------------------------------------------------
    for (size_t k = 0; k <= 1; ++k) {
      string value_t = (k==0) ? "vector<point_basic<T> >&" : "typename vector<point_basic<T> >::iterator";
      out << "template<class T>" << endl
          << "void" << endl
          << class_name.str() << "<T>::grad_eval(" << endl
          << "    reference_element        hat_K," << endl
          << "    const point_basic<T>&    hat_x," << endl
          << "    "<<value_t<<"            values) const" << endl
          << "{" << endl
          << "    switch (hat_K.variant()) {" << endl;
      for (size_type i = 0; i < reference_element::max_variant; i++) {
        const basis_symbolic_nodal_on_geo& b = operator[](i);
        if (b.size() == 0) continue;
        out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
            << "      return " << class_name.str() << "_" << b.hat_K().name() << "<T>::grad_eval (hat_x, values);" << endl
            << "      }" << endl;
      }
      out << "      default : {" << endl
          << "        error_macro (\"grad_eval: unsupported `\" << hat_K.name() << \"' element type\");" << endl
          << "      }" << endl
          << "    }" << endl
          << "}" << endl;
    }
    // --------------------------------------------------
    // hat_node (vectorial)
    // --------------------------------------------------
    for (size_t k = 0; k <= 1; ++k) {
      string value_t = (k==0) ? "vector<point_basic<T> >&" : "typename vector<point_basic<T> >::iterator";
      out << "template<class T>" << endl
          << "void" << endl
          << class_name.str() << "<T>::hat_node(" << endl
          << "    reference_element hat_K," << endl
          << "    "<<value_t<<"     hat_node) const" << endl
          << "{" << endl
          << "    switch (hat_K.variant()) {" << endl;
      for (size_type i = 0; i < reference_element::max_variant; i++) {
        const basis_symbolic_nodal_on_geo& b = operator[](i);
        if (b.size() == 0) continue;
        out << "      case reference_element::" << b.hat_K().name() << ": {" << endl
            << "        return " << class_name.str() << "_" << b.hat_K().name() << "<T>::hat_node (hat_node);" << endl
            << "      }" << endl;
      }
      out << "      default : {" << endl
          << "        error_macro (\"hat_node: unsupported `\" << hat_K.name() << \"' element type\");" << endl
          << "      }" << endl
          << "    }" << endl
          << "}" << endl;
    }
    // --------------------------------------------------
    // call to a constructor
    // --------------------------------------------------
    out << "// instantiation in library:" << endl
	<< "template class basis_" << name() << "<Float>;" << endl
        << "} // namespace rheolef" << endl;
}
void 
basis_symbolic_nodal::put_cxx_main (int argc, char**argv) const
{
    if (argc <= 1 || string(argv[1]) == "-h") {
      put_cxx_header (cout);
    } else {
      put_cxx_body (cout);
    }
}
